#pragma once
#include <BasicAbstractions/IState.h>
#include <SFML/Graphics.hpp>
#include <States/GameState/GameState.h>

struct IGameBuilder {
    virtual void create_rooms() = 0;
    virtual void set_rooms_side() = 0;
    virtual void create_context(float dynamic_objets_ratio) = 0;
    virtual void create_state(IStateManager& state_manager, const sf::VideoMode& mode, const std::string& window_title) = 0;
    virtual void set_all_to_state() = 0;
    virtual std::unique_ptr<GameState> get_game() = 0; ///< @todo unique_ptr?
    virtual ~IGameBuilder() = default;
};

class CommonBuilder : public IGameBuilder {
public:
    CommonBuilder(const float width, const float height, const float room_size) : m_width(width), m_height(height), m_room_size(room_size) {}
    void create_context(float dynamic_objets_ratio) override;
    void create_state(IStateManager& state_manager, const sf::VideoMode& mode, const std::string& window_title) override;
    void set_all_to_state() override;
    std::unique_ptr<GameState> get_game() override { return std::move(m_game_state); }
private:
    float m_width;
    float m_height;
    float m_room_size;
    std::vector<std::vector<std::unique_ptr<Room>>> m_rooms;
    GameContext m_context;
    std::unique_ptr<GameState> m_game_state;
};

struct SimpleBuilder : CommonBuilder {
    using CommonBuilder::CommonBuilder; ///< @todo ???
    void create_rooms() override {}
    void set_rooms_side() override {}
};

struct ComplexBuilder : CommonBuilder {
    using CommonBuilder::CommonBuilder; ///< @todo ???
    void create_rooms() override {}
    void set_rooms_side() override {}
};